$primary-selection-color: #43B02A;
$primary-selection-color-alt-1: #006038;
$primary-selection-color-alt-2: #007041;
$primary-selection-color-alt-3: #A1D8A4;
$primary-selection-color-alt-4: #69C06D;

$primary-text-color: #000000;
$primary-text-color-alt-1: #4C4C4C;
$primary-text-color-alt-2: #6E7C7C;
$primary-text-color-alt-2: #AAAAAA;


$secondary-text-color: #FFFFFF;

$primary-background-color: #FFFFFF;
$primary-background-color-alt-1: #F5F5F5;
$primary-background-color-alt-2: #E8E8E8;
$primary-background-color-alt-3: #DDDDDD;
$primary-background-color-alt-4: #D8D8D8;
$primary-background-color-alt-5: #CCCCCC;
$primary-background-color-alt-6: #BBBBBB;
$primary-background-color-alt-7: #808080;

$secondary-background-color: #006038;

$progressbar-color: #00A3E0;

$STYLED_ICONS_COLOR: '#000000';

@import 'common';